#ifndef _DEV_AT73DAC_H_
#define _DEV_AT73DAC_H_

/*
 * Copyright (C) 2006 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*!
 * \file at73dac.h
 * \brief AT73C213 driver.
 *
 * \verbatim
 *
 * $Log$
 *
 * \endverbatim
 */

#include <sys/types.h>

#if defined(AT91SAM9260_EK)
#define SAMPLE_RATE 44100
//#define SAMPLE_RATE 32000
//#define SAMPLE_RATE 16000
#else
#define SAMPLE_RATE 8100
#endif

#define DAC_CTRL    0x00    /*!< \brief DAC control register. */
#define DAC_CTRL_ONLNIL		0x01    /*!< \brief Left channel line in amp. power up. */
#define DAC_CTRL_ONLNIR		0x02    /*!< \brief Right channel line in amp. power up. */
#define DAC_CTRL_ONLNOL		0x04    /*!< \brief Left channel line out driver power up. */
#define DAC_CTRL_ONLNOR		0x08    /*!< \brief Right channel line out driver power up. */
#define DAC_CTRL_ONDACL		0x10    /*!< \brief Left channel DAC power up. */
#define DAC_CTRL_ONDACR		0x20    /*!< \brief Right channel DAC power up. */
#define DAC_CTRL_ONAUXIN	0x40    /*!< \brief Diff. mono aux. input amp. power up. */
#define DAC_CTRL_ONPADRV	0x80    /*!< \brief Diff. mono PA driver power up. */

#define DAC_LLIG    0x01    /*!< \brief DAC left line in gain register. */

#define DAC_RLIG    0x02    /*!< \brief DAC right line in gain register. */

#define DAC_LMPG    0x03    /*!< \brief DAC left master playback gain register. */

#define DAC_RMPG    0x04    /*!< \brief DAC right master playback gain register. */

#define DAC_LLOG    0x05    /*!< \brief DAC left line out gain register. */

#define DAC_RLOG    0x06    /*!< \brief DAC right line out gain register. */

#define DAC_OLC     0x07    /*!< \brief DAC output level control register. */
#define DAC_OLC_LOLC		0x07    /*!< \brief Left channel output level control selector mask. */
#define DAC_OLC_LOLC_LSB	    0   /*!< \brief Left channel output level control selector LSB. */
#define DAC_OLC_LSHORT		0x08    /*!< \brief Left channel short circuit indicator. */
#define DAC_OLC_ROLC		0x70    /*!< \brief Right channel output level control selector mask. */
#define DAC_OLC_ROLC_LSB	    4   /*!< \brief Right channel output level control selector LSB. */
#define DAC_OLC_RSHORT		0x80    /*!< \brief Right channel short circuit indicator. */

#define DAC_MC      0x08    /*!< \brief DAC mixer control register. */
#define DAC_MC_LMSMIN1		0x01    /*!< \brief Left channel mono/stereo mixer left mixed input enable. */
#define DAC_MC_LMSMIN2		0x02    /*!< \brief Left channel mono/stereo mixer right mixed input enable. */
#define DAC_MC_RMSMIN1		0x04    /*!< \brief Right channel mono/stereo mixer left mixed input enable. */
#define DAC_MC_RMSMIN2		0x08    /*!< \brief Right channel mono/stereo mixer right mixed input enable. */
#define DAC_MC_INVL		    0x10    /*!< \brief Left channel mixer output invert. */
#define DAC_MC_INVR		    0x20    /*!< \brief Right channel mixer output invert. */

#define DAC_CSFC    0x09    /*!< \brief DAC clock and sampling frequency control register. */
#define DAC_CSFC_OVRSEL		0x10    /*!< \brief Master clock selector. Set for 384 * Fs, otherwise 256 * Fs */

#define DAC_MISC    0x0A    /*!< \brief DAC miscellaneous register. */
#define DAC_MISC_NBITS      0x03    /*!< \brief Data interface word length mask. */
#define DAC_MISC_NBITS_16   0x00   /*!< \brief 16-bit samples. */
#define DAC_MISC_NBITS_18   0x01   /*!< \brief 18-bit samples. */
#define DAC_MISC_NBITS_20   0x02   /*!< \brief 20-bit samples. */
#define DAC_MISC_DEEMPEN	0x04    /*!< \brief De-emphasis enable. */
#define DAC_MISC_DITHEN		0x08    /*!< \brief Dither enable. */
#define DAC_MISC_DINTSEL    0x30    /*!< \brief Data format mode selector mask. */
#define DAC_MISC_DINTSEL_I2S 0x00   /*!< \brief I2S justified. */
#define DAC_MISC_DINTSEL_MSB 0x10   /*!< \brief MSB justified. */
#define DAC_MISC_DINTSEL_LSB 0x20   /*!< \brief LSB justified. */
#define DAC_MISC_VCMCAPSEL	0x80    /*!< \brief VCM decoupling capacitor selector. */

#define DAC_PRECH   0x0C    /*!< \brief DAC pre-charge control register. */
#define DAC_PRECH_ONMSTR	0x01    /*!< \brief Master power on control. */
#define DAC_PRECH_PRCHG		0x02    /*!< \brief Master pre-charge. */
#define DAC_PRECH_PRCHGLNIL	0x04    /*!< \brief Left channel line in pre-charge. */
#define DAC_PRECH_PRCHGLNIR	0x08    /*!< \brief Right channel line in pre-charge. */
#define DAC_PRECH_PRCHGLNOL	0x10    /*!< \brief Left channel line out pre-charge. */
#define DAC_PRECH_PRCHGLNOR	0x20    /*!< \brief Right channel line out pre-charge. */
#define DAC_PRECH_PRCHGAUX1	0x40    /*!< \brief Differential mono auxiliary input pre-charge. */
#define DAC_PRECH_PRCHGPDRV	0x80    /*!< \brief Differential mono PA driver pre-charge. */

#define DAC_AUXG    0x0D    /*!< \brief DAC auxiliary input gain control. */

#define DAC_RST     0x10    /*!< \brief DAC reset register. */
#define DAC_RST_RSTZ		0x01    /*!< \brief Active low reset of the audio codec. */
#define DAC_RST_RESFILZ		0x02    /*!< \brief Active low reset of the audio codec filter. */
#define DAC_RST_RESMASK		0x04    /*!< \brief Active high reset mask of the audio codec. */

#define PA_CTRL     0x11    /*!< \brief Power amplifier control register. */
#define PA_CTRL_APAGAIN     0x0F    /*!< \brief Amplifier gain mask. */
#define PA_CTRL_APAGAIN_LSB     0   /*!< \brief Amplifier gain LSB. */
#define PA_CTRL_APALP       0x10    /*!< \brief Low power mode. */
#define PA_CTRL_APAPRECH    0x20    /*!< \brief Precharge. */
#define PA_CTRL_APAON       0x40    /*!< \brief Activate. */

#define DAC_RREG(r)     (((r) | 0x80) << 8) /*!< \brief Read register with 16 bit SPI mode. */
#define DAC_WREG(r, v)  (((r) << 8) | (v))  /*!< \brief Write register with 16 bit SPI mode. */

__BEGIN_DECLS
/* Prototypes */

extern int At73DacInit(void);
extern int At73I2sDisable(void);
extern int At73I2sEnable(u_int rate);
extern int At73DacAdaptMasterClock(u_int rate);
extern int At73DacWrite(void *buf, int len);
extern int At73DacStart(void);
extern int At73DacFlush(void);

extern u_char At73DacReadReg(u_int reg);
extern void At73DacWriteReg(u_int reg, u_int val);

extern int At73DacSetVolume(int left, int right);

__END_DECLS
/* End of prototypes */
#endif
